import React, { useEffect, useState } from 'react';
import styled from 'styled-components';

const PassCodeInput = ({ type, fields, correctPassword }) => {
    const INPUTS_NUM = fields || 4;

    const [focused, setFocused] = useState(0);
    let [inputValue, setInputValue] = useState([]);
    const [inputElements, setInputElements] = useState([]);

    useEffect(() => {
        const parent = document.getElementById('pass-code-input-wrapper');
        setInputElements(parent.children);
        parent.children[0].focus();
    }, []);
    useEffect;

    const removeInputValue = (elem) => {
        elem.removeAttribute('value');
        elem.value = '';
    };

    const makeFocused = (elem, id) => {
        elem.focus();
        setFocused(id);
    };

    useEffect(() => {
        if (focused === fields) {
            const validCode = correctPassword(inputValue.join(''));
            if (!validCode) {
                setTimeout(() => {
                    setInputValue([]);
                    const parent = document.getElementById(
                        'pass-code-input-wrapper'
                    );
                    setFocused(0);
                    setInputElements(parent.children);

                    for (let child of parent.children) {
                        child.value = '';
                    }

                    parent.children[0].focus();
                }, 300);
            }
        }
    }, [focused]);

    const onInputChange = async (e) => {
        if (e.target.value.length > 1) {
            e.target.value = e.target.value[0];
            e.preventDefault();
            return;
        }
        if (focused < INPUTS_NUM - 1) {
            let currentFocus = focused;

            makeFocused(e.target.nextSibling, currentFocus + 1);
            setInputValue((inputValue) => [...inputValue, e.target.value]);
        } else if (focused === INPUTS_NUM - 1) {
            await setInputValue((inputValue) => [
                ...inputValue,
                e.target.value,
            ]);
            //Remove if wanted to manipulate after entering last char
            inputElements[focused].blur();
            inputElements[focused].classList.add('full');
            setFocused(4);
        }
    };

    const handleKeyDown = (event) => {
        if (event.keyCode === 8 || event.keyCode === 46) {
            const previousSiblingId = focused - 1;

            const newInput = inputValue;
            newInput.pop();
            setInputValue(newInput);

            if (focused === INPUTS_NUM - 1) {
                if (event.target.value === '') {
                    makeFocused(
                        inputElements[previousSiblingId],
                        previousSiblingId
                    );
                    removeInputValue(inputElements[previousSiblingId]);
                }
                inputElements[focused].classList.remove('full');
                removeInputValue(inputElements[focused]);
                event.target.value = '';
            } else if (previousSiblingId >= 0) {
                removeInputValue(inputElements[previousSiblingId]);
                removeInputValue(inputElements[focused]);
                makeFocused(
                    inputElements[previousSiblingId],
                    previousSiblingId
                );
                event.target.value = '';
            } else {
                makeFocused(inputElements[0], 0);
                removeInputValue(inputElements[1]);
                removeInputValue(inputElements[0]);

                setInputValue([]);
            }

            event.preventDefault();
        }
    };

    return (
        <>
            {inputValue !== undefined && (
                <Wrapper id='pass-code-input-wrapper'>
                    {[...Array(INPUTS_NUM)].map((e, i) => (
                        <InputField
                            type={type}
                            className={inputValue[i] ? 'full' : ''}
                            value={inputValue[i]}
                            onChange={onInputChange}
                            onKeyDown={handleKeyDown}
                        />
                    ))}
                </Wrapper>
            )}
        </>
    );
};

const Wrapper = styled.div`
    display: flex;
    flex-direction: row;
`;

const InputField = styled.input`
    margin-right: 20px;
    width: 15px;
    height: 15px;
    border-radius: 50%;
    outline: none;
    border: none;
    text-align: center;
    border: none;
    margin-right: 26px;
    outline: none;
    transition: 500ms;
    border-radius: 50px;
    user-select: none;
    background: transparent;
    border: 2px solid #adece1;
    color: transparent;
    /* background: white;
    color: transparent; */

    &:last-child {
        margin-right: 0px;
    }

    &:focus {
        outline: none !important;

        /* color: transparent !important;
        background: #dee3e4 !important; */

        background: transparent !important;
        border: 2px solid #adece1 !important;
    }

    &.full {
        background-color: white !important;
        color: transparent !important;
        border: 2px solid white;
    }

    &::-webkit-outer-spin-button,
    &::-webkit-inner-spin-button {
        -webkit-appearance: none;
        margin: 0;
    }

    &:-moz-selection {
        color: transparent;
        background: transparent;
    }

    &[type='number'] {
        -moz-appearance: textfield;
    }
`;

export default PassCodeInput;
